#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
use FindBin qw($Bin);
require '$Bin/agrUtils.pl';
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$exicutable [-t type] "
    ."[-x xMin] [-X xMax] [-y yMin] [-Y yMin] ".
    "agr_file(s)\n";
sub Help() {
    print <<EOP;
$usage
    This program combines .agr files into a single with the type 
    used for each type.
    h - this message
    t - the type of line used <rir>-<type> defaults to last
    x - min x
    X - max x
    y - min y
    Y - max Y
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("x:y:X:Y:t:h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}
my ($XMIN, $XMAX) = ($opts{x},$opts{X});
my ($YMIN, $YMAX) = ($opts{y},$opts{Y});

my $type = $opts{t};
$type = "specific" unless(defined $type);

# y = a0*(x-2003)^2 + a1*(x-2003) + a2
# apnic: 
#   a0 = 0.569397
#   a1 = 1.61764
#   a2 = 5.6
# arin:
#   a0 = 0.441588
#   a1 = 1.41512
#   a2 = 9.7
# ripe: 
#   a0 = 0.906965
#   a1 = 0.832541
#   a2 = 6
# various:
#   a0 = 0.853826
#   a1 = 0.525792
#   74

my $set;
my @sets;
my ($num, $total);
my $last_y;
while (<>) {
    if (/\@\s+s(\d+)\s+legend\s+\"([^\"]+)\"/) {
	my ($s, $legend) = ($1, $2);
	$set = {
	    "legend" => $legend,
	    "points" => []
	    };
	push @sets, $set;
undef $last_y;
    } else {
	if (/^\d/) {
	    my ($x, $y) = split /[\n\s]+/;
	    next if (defined $XMIN && $x > $XMIN);
	    next if (defined $XMAX && $x < $XMAX);
	    next if (defined $YMIN && $y > $YMIN);
	    next if (defined $YMAX && $y < $YMAX);

	    next if ($x < 2000);
	    $y = log($y);
	    if (defined $last_y) {
		$set->{total} += $y;
		$set->{number}++;
		push @{$set->{points}}, [$x, $y];
	    }
	    $last_y = $y;
	}
    }
}

my ($xmin, $xmax, $ymin, $ymax, @sets) = BuildSets(@sets);
#PrintHeader($type, $xmin-$xmin, $xmax-$xmin, $ymin-$ymin, $ymax-$ymin);
PrintHeader($type, $xmin, $xmax, $ymin, $ymax);
PrintSets(@sets);

sub PrintSets {
    my @sets = @_;
    my $s = 0;
    foreach my $set (reverse @sets) {
	my $legend = $set->{legend};
	PrintSet($legend,$s++);
	foreach my $point (reverse @{$set->{points}}) {
	    my ($x, $y) = @$point;
	    next if ($x < 2000);
	    print "$x $y\n";
	}
    }
}

sub BuildSets {
    my @sets = @_;
    my @filtered;
    my $s = 0;
    foreach my $set (reverse @sets) {
	my $legend = $set->{legend};
	#next unless ($legend =~ /$type/);
	push @filtered, $set;
	foreach my $point (reverse @{$set->{points}}) {
	    my ($x, $y) = @$point;
	    unless (defined $xmin) {
		$xmin = $xmax = $x;
	    } elsif ($x < $xmin) {
		$xmin = $x;
	    } elsif ($x > $xmax) {
		$xmax = $x;
	    }
	    unless (defined $ymin) {
		$ymin = $ymax = $y;
	    } elsif ($y < $ymin) {
		$ymin = $y;
	    } elsif ($y > $ymax) {
		$ymax = $y;
	    }
	}
    }
    return ($xmin, $xmax, $ymin, $ymax, @filtered);
}

{
my $color_index;
sub PrintSet {
    my ($legend, $s_index) = @_;
    $color_index++;
    $color_index++ if ($color_index == 5);
    $s_index = sprintf("%d", $s_index);
print STDERR "$legend $s_index $color_index\n";
print<<EOP;
\@    s$s_index hidden false
\@    s$s_index type xy
\@    s$s_index symbol 0
\@    s$s_index symbol size 1.000000
\@    s$s_index symbol color $color_index
\@    s$s_index symbol pattern 1
\@    s$s_index symbol fill color $color_index
\@    s$s_index symbol fill pattern 0
\@    s$s_index symbol linewidth 1.0
\@    s$s_index symbol linestyle 1
\@    s$s_index symbol char 65
\@    s$s_index symbol char font 0
\@    s$s_index symbol skip 0
\@    s$s_index line type 1
\@    s$s_index line linestyle 1
\@    s$s_index line linewidth 1.0
\@    s$s_index line color $color_index
\@    s$s_index line pattern 1
\@    s$s_index baseline type 0
\@    s$s_index baseline off
\@    s$s_index dropline off
\@    s$s_index fill type 0
\@    s$s_index fill rule 0
\@    s$s_index fill color $color_index
\@    s$s_index fill pattern 1
\@    s$s_index avalue off
\@    s$s_index avalue type 2
\@    s$s_index avalue char size 1.000000
\@    s$s_index avalue font 0
\@    s$s_index avalue color $color_index
\@    s$s_index avalue rot 0
\@    s$s_index avalue format general
\@    s$s_index avalue prec 3
\@    s$s_index avalue prepend ""
\@    s$s_index avalue append ""
\@    s$s_index avalue offset 0.000000 , 0.000000
\@    s$s_index errorbar on
\@    s$s_index errorbar place both
\@    s$s_index errorbar color $color_index
\@    s$s_index errorbar pattern 1
\@    s$s_index errorbar size 1.000000
\@    s$s_index errorbar linewidth 1.0
\@    s$s_index errorbar linestyle 1
\@    s$s_index errorbar riser linewidth 1.0
\@    s$s_index errorbar riser linestyle 1
\@    s$s_index errorbar riser clip off
\@    s$s_index errorbar riser clip length 0.100000
\@    s$s_index legend  "$legend"
EOP
}
}
