#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
use FindBin qw($Bin);
require "$Bin/agrUtils.pl";
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$exicutable [-t type] "
    ."[-x xMin] [-X xMax] [-y yMin] [-Y yMin] ".
    "agr_file(s)\n";
sub Help() {
    print <<EOP;
$usage
    This program combines .agr files into a single with the type 
    used for each type.
    h - this message
    t - the type of line used <rir>-<type> defaults to last
    x - min x
    X - max x
    y - min y
    Y - max Y
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("x:y:X:Y:t:h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}
my ($XMIN, $XMAX) = ($opts{x},$opts{X});
my ($YMIN, $YMAX) = ($opts{y},$opts{Y});

my $type = $opts{t};

my $set;
my @sets;
my ($num, $total);
while (<>) {
    if (/\@\s+s(\d+)\s+legend\s+\"([^\"]+)\"/) {
	my ($s, $legend) = ($1, $2);
	if (defined $set) {
	    push @sets, $set;
	}
	$set = {
	    "legend" => $legend,
	    "points" => []
	    };
    } else {
	if (/^\d/) {
	    my ($x, $y) = split /[\n\s]+/;
	    $set->{total} += $y;
	    $set->{number}++;
	    push @{$set->{points}}, [$x, $y];
	}
    }
}
push @sets, $set;

my %rir2type2set = GroupByRIR(@sets);
my ($xmin, $xmax, $ymin, $ymax, @sets) = BuildDiffSets(%rir2type2set);
$xmin = $XMIN if (defined $XMIN);
$xmax = $XMAX if (defined $XMAX);
$ymin = $YMIN if (defined $YMIN);
$ymax = $YMAX if (defined $YMAX);
PrintHeader($xmin, $xmax, $ymin, $ymax, $type);
PrintSets(@sets);

sub GroupByRIR {
    my @sets = @_;
    my %rir2type2set;
    foreach my $set (@sets) {
	if ($set->{legend} =~ /([^-]+)-(.+)$/) {
	    my ($rir, $type) = ($1, $2);
	    $rir2type2set{$rir}{$type} = $set;
	}
    }
    return %rir2type2set;
}

sub BuildDiffSets {
    my %rir2type2set = @_;
    my @diff_sets;
    foreach my $rir (keys %rir2type2set) {
	next unless (defined $rir2type2set{$rir}{"IANA"}{"points"}
	    && $rir2type2set{$rir}{"most specific"}{"points"});

	my @iana = reverse sort {$a->[0]<=>$b->[0]}
		@{$rir2type2set{$rir}{"IANA"}{"points"}};
	my @specific = reverse sort {$a->[0]<=>$b->[0]}
		@{$rir2type2set{$rir}{"most specific"}{"points"}};
	my @points;
	my $total;
	while ($#iana >= 0 && $#specific >= 0) {
	    my $x;
	    my $y = $iana[0][1] - $specific[0][1];
	    if ($iana[0][0] > $specific[0][0]) {
		$x = $iana[0][0];
		shift @iana;
	    } elsif ($iana[0][0] < $specific[0][0]) {
		$x = $specific[0][0];
		shift @specific;
	    } elsif ($iana[0][0] == $specific[0][0]) {
		$x = $specific[0][0];
		shift @iana;
		shift @specific;
	    } else {
		die("trapped in loop!");
	    }
	    if ((!defined $XMIN || $x >= $XMIN) &&
		(!defined $XMAX || $x <= $XMAX) &&
		(!defined $YMIN || $y >= $YMIN) &&
		(!defined $YMAX || $y <= $YMAX)) {
		$total += $y;
		push @points, [$x, $y];
		unless (defined $xmin) {
		    $xmin = $xmax = $x;
		} elsif ($x < $xmin) {
		    $xmin = $x;
		} elsif ($x > $xmax) {
		    $xmax = $x;
		}
		unless (defined $ymin) {
		    $ymin = $ymax = $y;
		} elsif ($y < $ymin) {
		    $ymin = $y;
		} elsif ($y > $ymax) {
		    $ymax = $y;
		}
	    }
	}
	if ($#points >= 0) {
	    push @diff_sets, {
		"legend"=>$rir, 
		"total" => $total,
		"number" => ($#points+1),
		"points"=>\@points
		};
	}
    }
    $xmin = $XMIN if (defined $XMIN);
    $xmax = $XMAX if (defined $XMAX);
    $ymin = $YMIN if (defined $YMIN);
    $ymax = $YMAX if (defined $YMAX);
    return ($xmin, $xmax, $ymin, $ymax, @diff_sets);
}

sub PrintSets {
    my @sets = @_;
    my @sorted = reverse sort {
	    $a->{total}/$a->{number}
		<=>
	    $b->{total}/$b->{number}
	    } @sets;
    my $s = 0;
    foreach my $set (reverse @sorted) {
	my $legend = $set->{legend};
	PrintSet($legend,$s++);
	foreach my $point (reverse @{$set->{points}}) {
	    my ($x, $y) = @$point;
	    print "$x $y\n";
	}
    }
}

{
my $color_index;
sub PrintSet {
    my ($legend, $s_index) = @_;
    $color_index++;
    $color_index++ if ($color_index == 5);
    $s_index = sprintf("%d", $s_index);
print STDERR "$legend $s_index $color_index\n";
print<<EOP;
\@    s$s_index hidden false
\@    s$s_index type xy
\@    s$s_index symbol 0
\@    s$s_index symbol size 1.000000
\@    s$s_index symbol color $color_index
\@    s$s_index symbol pattern 1
\@    s$s_index symbol fill color $color_index
\@    s$s_index symbol fill pattern 0
\@    s$s_index symbol linewidth 1.0
\@    s$s_index symbol linestyle 1
\@    s$s_index symbol char 65
\@    s$s_index symbol char font 0
\@    s$s_index symbol skip 0
\@    s$s_index line type 1
\@    s$s_index line linestyle 1
\@    s$s_index line linewidth 1.0
\@    s$s_index line color $color_index
\@    s$s_index line pattern 1
\@    s$s_index baseline type 0
\@    s$s_index baseline off
\@    s$s_index dropline off
\@    s$s_index fill type 0
\@    s$s_index fill rule 0
\@    s$s_index fill color $color_index
\@    s$s_index fill pattern 1
\@    s$s_index avalue off
\@    s$s_index avalue type 2
\@    s$s_index avalue char size 1.000000
\@    s$s_index avalue font 0
\@    s$s_index avalue color $color_index
\@    s$s_index avalue rot 0
\@    s$s_index avalue format general
\@    s$s_index avalue prec 3
\@    s$s_index avalue prepend ""
\@    s$s_index avalue append ""
\@    s$s_index avalue offset 0.000000 , 0.000000
\@    s$s_index errorbar on
\@    s$s_index errorbar place both
\@    s$s_index errorbar color $color_index
\@    s$s_index errorbar pattern 1
\@    s$s_index errorbar size 1.000000
\@    s$s_index errorbar linewidth 1.0
\@    s$s_index errorbar linestyle 1
\@    s$s_index errorbar riser linewidth 1.0
\@    s$s_index errorbar riser linestyle 1
\@    s$s_index errorbar riser clip off
\@    s$s_index errorbar riser clip length 0.100000
\@    s$s_index legend  "$legend"
EOP
}
}
