#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
use Getopt::Std;
use FindBin qw($Bin);
use vars qw(%type2rir2prediction);
require "$Bin/agrUtils.pl";
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$exicutable -t type "
    ."[-x xMin] [-X xMax] [-y yMin] [-Y yMin] ".
    "agr_file(s)\n";
sub Help() {
    print <<EOP;
$usage
    This program combines .agr files into a single with the type 
    used for each type.
    h - this message
    t - the type of line used <rir>-<type> defaults to last
    x - min x
    X - max x
    y - min y
    Y - max Y
EOP
}

# y = a0*(x-2003)^2 + a1*(x-2003) + a2
# apnic: 
#   a0 = 0.569397
#   a1 = 1.61764
#   a2 = 5.6
# arin:
#   a0 = 0.441588
#   a1 = 1.41512
#   a2 = 9.7
# ripe: 
#   a0 = 0.906965
#   a1 = 0.832541
#   a2 = 6
# various:
#   a0 = 0.853826
#   a1 = 0.525792
#   74

my %opts;
unless (getopts("x:y:X:Y:t:h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}
my ($XMIN, $XMAX) = ($opts{x},$opts{X});
my ($YMIN, $YMAX) = ($opts{y},$opts{Y});

my $type = $opts{t};
unless (defined $type) {
    die("You must defined type");
} 

my $rir2prediction = GetRirPrediction($type);
unless (defined $rir2prediction) {
    die("Type:$type is not first or specific");
} 
my %rir2prediction = %{$rir2prediction};

my $set;
my @sets;
my ($num, $total);
my $linenum = 0;
my ($xmin,$xmax, $ymin, $ymax);
print STDERR "$xmin,$xmax\n";
while (<>) {
    $linenum++;
    if (/\@\s+s(\d+)\s+legend\s+\"([^\"]+)\"/) {
	my ($s, $legend) = ($1, $2);
	AddSet($set);
	$set = {
	    "legend" => $legend,
	    "points" => []
	    };
    } else {
	if (/^\d/) {
	    my ($x, $y) = split /[\n\s]+/;

            next if (defined $XMIN && $x < $XMIN);
            next if (defined $XMAX && $x > $XMAX);
            next if (defined $YMIN && $y < $YMIN);
            next if (defined $YMAX && $y > $YMAX);
	    $set->{total} += $y;
	    $set->{number}++;
	    if (!defined $xmin) {
		$xmin = $xmax = $x;
	    } elsif ($xmin > $x) {
		$xmin = $x;
	    }elsif ($xmax < $x) {
		$xmax = $x;
	    }
	    if (!defined $ymin) {
		$ymin = $ymax = $y;
	    } elsif ($ymin > $y) {
		$ymin = $y;
	    }elsif ($ymax < $y) {
		$ymax = $y;
	    }
	    push @{$set->{points}}, [$x, $y];
	}
    }
}
AddSet($set);

sub AddSet {
    my ($set) = @_;
    if (defined $set && $set->{number} > 0 
	&& ( $set->{legend} =~ /specific/ || $set->{legend} =~ /IANA/)) {
	push @sets, $set;
    }
}

PrintSets(@sets);

sub GroupByRIR {
    my @sets = @_;
    my %rir2type2set;
    foreach my $set (@sets) {
	if ($set->{legend} =~ /([^-]+)-(.+)$/) {
	    my ($rir, $type) = ($1, $2);
	    $rir2type2set{$rir}{$type} = $set;
	}
    }
    return %rir2type2set;
}

sub PrintSets {
    my @sets = @_;
    my @sorted = reverse sort {
	    $a->{total}/$a->{number}
		<=>
	    $b->{total}/$b->{number}
	    } @sets;
    my $s = 0;
    my %rir2specific;
    # Add extrea points to last
    my $last_x;
    foreach my $set (reverse @sorted) {
	my $legend = $set->{legend};
	my ($rir, $type) = split /\-/, $legend;
	next unless ($type =~ "specific");
	$rir2specific{$rir} = $set;

	my @points = reverse sort {$a->[0]<=>$b->[0]} @{$set->{points}};
	$last_x = $points[0][0];
	if (defined $rir2prediction{$rir}) {
	    my $last_year = 2014;
	    my $number = 1000;
	    my $fraction = $last_year - $last_x;
	    my $equation = $rir2prediction{$rir}{"equation"};
	    my @a = @{$rir2prediction{$rir}{"a"}};
	    foreach my $i (0..$number) {
		my $x = ($i/$number)*$fraction + $last_x;
		my $y;
		eval $equation;
		push @{$set->{points}}, [$x, $y];
	    }
	}
    }
    # Add extrea points to IANA
    foreach my $set (reverse @sorted) {
	my $index_18m = 0; # pointer to the last 18 months
	my $legend = $set->{legend};
	my ($rir, $type) = split /\-/, $legend;
	next unless ($type eq "IANA");
	my $num_points = @{$set->{points}};
	my $last_y = $set->{points}[$num_points-1][1];
	if (defined $rir2specific{$rir} && defined $rir2prediction{$rir}) {
	    my ($limit, $jump) = @{$rir2prediction{$rir}{"pool"}};
	    my @points = @{$rir2specific{$rir}{"points"}};
	    my $pool_y = $last_y;
	    foreach my $index (0..$#points) {
		my ($x, $y) = @{$points[$index]};
		my $diff = 0;
    
		while ($index_18m < $#points && 
		    $points[$index_18m][0] < $x-1.8) {
		    $index_18m++;
		}
		next if ($x <= $last_x);
		if ($rir ne "various") {
		    my $jump = 0;
		    my $i = $index_18m;
		    while ($i <= $index) {
			my $diff = ($points[$i][1] - $points[$i-1][1]);
			$jump += $diff;
			$i++;
		    }
		    if ($jump > 3) {
			$jump = 3;
		    }
		    if ($pool_y-$y < $limit) {
			$pool_y += $jump;
			push @{$set->{points}}, [$x, $pool_y];
		    }
		} else {
		    if ($pool_y <= $y) {
			$points[$index][1] = $pool_y;
		    }
		    push @{$set->{points}}, [$x, $pool_y];
		}
		if (!defined $xmin) {
		    $xmin = $xmax = $x;
		} elsif ($xmin > $x) {
		    $xmin = $x;
		}elsif ($xmax < $x) {
		    $xmax = $x;
		}
		if (!defined $ymin) {
		    $ymin = $ymax = $y;
		} elsif ($ymin > $y) {
		    $ymin = $y;
		}elsif ($ymax < $y) {
		    $ymax = $y;
		}
	    }
	}
    }

    $xmin = $XMIN if (defined $XMIN && $xmin < $XMIN);
    $xmax = $XMAX if (defined $XMAX && $xmax > $XMAX);
    $ymax = $YMAX if (defined $YMAX && $ymax > $YMAX);

    PrintHeader($xmin, $xmax, $ymin, $ymax, $type);
    print "# type:$type\n";
    print "# projection point:$last_x\n";
    foreach my $set (reverse @sorted) {
	my $legend = $set->{legend};
	my ($rir) = split /\-/, $legend;
	PrintSet($legend,$s++);
	foreach my $point (@{$set->{points}}) {
	    my ($x, $y) = @$point;
	    print "$x $y\n";
	}
    }
}

{
my $color_index;
sub PrintSet {
    my ($legend, $s_index) = @_;
    $color_index++;
    $color_index++ if ($color_index == 5);
    $s_index = sprintf("%d", $s_index);
print<<EOP;
\@    s$s_index hidden false
\@    s$s_index type xy
\@    s$s_index symbol 0
\@    s$s_index symbol size 1.000000
\@    s$s_index symbol color $color_index
\@    s$s_index symbol pattern 1
\@    s$s_index symbol fill color $color_index
\@    s$s_index symbol fill pattern 0
\@    s$s_index symbol linewidth 1.0
\@    s$s_index symbol linestyle 1
\@    s$s_index symbol char 65
\@    s$s_index symbol char font 0
\@    s$s_index symbol skip 0
\@    s$s_index line type 1
\@    s$s_index line linestyle 1
\@    s$s_index line linewidth 1.0
\@    s$s_index line color $color_index
\@    s$s_index line pattern 1
\@    s$s_index baseline type 0
\@    s$s_index baseline off
\@    s$s_index dropline off
\@    s$s_index fill type 0
\@    s$s_index fill rule 0
\@    s$s_index fill color $color_index
\@    s$s_index fill pattern 1
\@    s$s_index avalue off
\@    s$s_index avalue type 2
\@    s$s_index avalue char size 1.000000
\@    s$s_index avalue font 0
\@    s$s_index avalue color $color_index
\@    s$s_index avalue rot 0
\@    s$s_index avalue format general
\@    s$s_index avalue prec 3
\@    s$s_index avalue prepend ""
\@    s$s_index avalue append ""
\@    s$s_index avalue offset 0.000000 , 0.000000
\@    s$s_index errorbar on
\@    s$s_index errorbar place both
\@    s$s_index errorbar color $color_index
\@    s$s_index errorbar pattern 1
\@    s$s_index errorbar size 1.000000
\@    s$s_index errorbar linewidth 1.0
\@    s$s_index errorbar linestyle 1
\@    s$s_index errorbar riser linewidth 1.0
\@    s$s_index errorbar riser linestyle 1
\@    s$s_index errorbar riser clip off
\@    s$s_index errorbar riser clip length 0.100000
\@    s$s_index legend  "$legend"
EOP
}
}
