#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use Time::Local;
use FindBin qw($Bin);
require "$Bin/agrUtils.pl";
use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$exicutable [-t type] agr_file(s)\n";
sub Help() {
    print <<EOP;
$usage
    This program combines .agr files into a single with the type 
    used for each type.
    h - this message
    t - the type of line used <rir>-<type> defaults to last
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("x:X:y:Y:i:t:h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my ($XMIN, $XMAX) = ($opts{x},$opts{X});
my ($YMIN, $YMAX) = ($opts{y},$opts{Y});

my $ieft_file;
if (defined $opts{i}) {
    $ieft_file = $opts{i};
}

my $set;
my @sets;
my ($num,$total);
my %xs;
my ($projection_point, $type);


my ($xmin, $xmax);
while (<>) {
    if (/#\s*projection point:\s*([^\s]+)/) {
        $projection_point = $1;
    }
    if (/#\s*type:\s*([^\s]+)/) {
        $type = $1;
    }
    if (/\@\s+s(\d+)\s+legend\s+\"([^\"]+)\"/) {
	my ($s, $type) = ($1, $2);
	if (defined $set) {
	    push @sets, $set;
	}
	my $legend = $type;
	$legend =~ s/-.*//;
	$set = {
	    "legend" => $legend,
	    "type" => $type,
	    "points" => []
	    };
    } else {
	if (/^\d/) {
	    my ($x, $y) = split /[\n\s]+/;
	    $set->{total} += $y;
	    $set->{number}++;
	    $xs{$x} = 0;	
	    push @{$set->{points}}, [$x, $y];

	    unless (defined $xmin) {
		$xmin = $xmax = $x;
	    } elsif ($x < $xmin) {
		$xmin = $total;
	    } elsif ($x > $xmax) {
		$xmax = $total;
	    }
	}
    }
}
push @sets, $set;

if (defined $ieft_file) {
    @sets = AddIETF($ieft_file, $xmin, $xmax, @sets);
}

@sets = AddPoolStripToLast(@sets);

my ($xmin, $xmax, $ymin, $ymax, @sorted) = AddStacked(@sets);

$xmin = $XMIN if (defined $XMIN && $xmin < $XMIN);
$xmax = $XMAX if (defined $XMAX && $xmax > $XMAX);
$ymin = $YMIN if (defined $YMIN && $ymin > $YMIN);
$ymax = $YMAX if (defined $YMAX && $ymax > $YMAX);

PrintHeader($xmin, $xmax, $ymin, $ymax, $type);
PrintSets(@sorted);

sub AddIETF {
    my ($file, $xmin, $xmax, @sets) = @_;
    open (IN, "<$file") || die("Unable to open $file:$!");
    my %sets;
    while (<IN>) {
	my ($block, $date, $type, $org, $name) = split /\s*\|\s*/;
	my $date_frac = DateFraction($date);
        if ($type ne "reserved" && $org =~ /IETF/) {
	    my $name = $org;
print STDERR "$date $type $name\n";
	    if ($name eq "IETF") {
		$name .= " special use";
	    }
            my ($length) = ($block =~ /\/(\d+)/);
            my $size;
            if ($length == 8) {
                $size = 1;
            } elsif ($length < 8) {
                $size = 1<<(8-$length);
            } else {
                $size = 1/(1<<($length-8));
            }
            if ($type eq "returned") {
                $size *= -1;
            }
	    $name = $name;
	    $sets{$name}{$date} += $size;
        }
    }
    close IN;
    foreach my $legend (keys %sets) {
	my $size = $sets{$legend};
	my @points;
	my $total = 0;
	foreach my $date (sort {$a<=>$b} keys %{$sets{$legend}}) {
	    my $size = $sets{$legend}{$date};
	    $total += $size;
	    push @points, [$date, $total];
	}
	push @points, [$xmax, $total];

	my $ieft_order;
	if ($legend =~ /future/) {
	    $ieft_order = 0;
	} elsif ($legend =~ /multicast/) {
	    $ieft_order = 1;
	} else {
	    $ieft_order = 2;
	}

	push @sets, {
	    "legend" => $legend,
	    "ieft order" => $ieft_order,
	    "type" => "$legend-IEFT",
	    "total" => $total,
	    "number" => 1,
	    "points" => \@points
	    };
    }
    return @sets;
}

sub DateFraction {
    my ($date) = @_;
    my ($year, $mon, $day) = ($date =~ /(\d{4})(\d\d)(\d\d)/);
    $day = "01" if ($day eq "00");
    $day = 28 if ((($year - 1980)%4 != 0) && $mon eq "02" && $day == 29);
    my ($s, $m, $h, $d, $m) = (0,0,0,1,0);
    my $start = Time::Local::timelocal($s,$m,$h,$d,$m,$year);
    my $current = Time::Local::timelocal($s,$m,$h,$day,$mon-1,$year);
    my $end = Time::Local::timelocal($s,$m,$h,$d,$m,$year+1);
    return $year + ($current-$start)/($end-$start);
}

sub AddPoolStripToLast {
    my @sets = @_;
    my @filtered;
    my %rir2pool;
    foreach my $set (@sets) {
	my ($rir, $class) = split /-/, $set->{type};
	if ($class eq "IANA") {
	    $rir2pool{$rir} = $set->{"points"};
	} elsif ($class =~ "specific") {
	    push @filtered, $set;
	} elsif ($class eq "IEFT") {
	    $rir2pool{$rir} = [];
	    push @filtered, $set;
	}
    }
    foreach my $set (@filtered) {
	my ($rir, $class) = split /-/, $set->{type};
	$set->{"pool"} = $rir2pool{$rir};
    } 
    return @filtered;
}

sub AddStacked {
    my @sets = @_;
    my @sorted = reverse sort {
	    my $a_ieft = $a->{type} =~ /IEFT/;
	    my $b_ieft = $b->{type} =~ /IEFT/;

	    if ($a_ieft && !$b_ieft) {
		return -1;
	    } elsif (!$a_ieft && $b_ieft) {
		return 1;
	    } elsif ($a_ieft && $b_ieft) {
		return $a->{"ieft order"} <=> $b->{"ieft order"};
	    }

	    $a->{total}/$a->{number}
		<=>
	    $b->{total}/$b->{number}
	} @sets;
    my @xs = reverse sort keys %xs;
    my %totals;
    foreach my $set (@sorted) {
	my $index = 0;
	my @stacked;
	my @stacked_pool;
	my @pool = reverse sort {$a->[0]<=>$b->[0]} @{$set->{"pool"}};
        my @point = reverse sort {$a->[0]<=>$b->[0]} @{$set->{"points"}};
	my ($pool_y, $point_y) = ($pool[0][1],$point[0][1]);
	foreach my $x (@xs) {
	    if ($pool[0][0] > $x) {
		shift @pool;
	    } 
	    if ($point[0][0] > $x) {
		shift @point;
	    } 
	    if ($#pool >= 0) {
		$pool_y = $pool[0][1];
	    } else {
		$pool_y = 0;
	    }
	    if ($#point >= 0) {
		$point_y = $point[0][1];
	    } else {
		$point_y = 0;
	    }
	    $totals{$x} += $point_y;
	    my $total = $totals{$x};
	    push @stacked, "$x $total";

	    if ($point_y < $pool_y) {
		$totals{$x} += ($pool_y - $point_y);
		$total = $totals{$x};
		push @stacked_pool, "$x $total";
	    }

	    unless (defined $xmin) {
		$xmin = $xmax = $x;
	    } elsif ($x < $xmin) {
		$xmin = $x;
	    } elsif ($x > $xmax) {
		$xmax = $x;
	    }
	    unless (defined $ymin) {
		$ymin = $ymax = $total;
	    } elsif ($total < $ymin) {
		$ymin = $total;
	    } elsif ($total > $ymax) {
		$ymax = $total;
	    }
	}
	$set->{stacked} = \@stacked;
	$set->{"stacked-pool"} = \@stacked_pool;
    }
    return ($xmin, $xmax, $ymin, $ymax, @sorted);
}

sub PrintSets {
    my @sorted = @_;
    my $s = 0;
    my $color_index = 2;
    foreach my $set (reverse @sorted) {
	my $legend = $set->{legend};
	my @stacked_pool = @{$set->{"stacked-pool"}};
	if ($#stacked_pool >= 0) {
	    PrintPool($color_index, $s++);
	    PrintPoints(@stacked_pool);
	}

	my @stacked = @{$set->{"stacked"}};
	if ($#stacked >= 0) {
	    PrintSet($legend, $color_index, $s++);
	    PrintPoints(@stacked);
	}
	$color_index++;
	$color_index++ if ($color_index == 5);
    }

    if (defined $projection_point) {
	PrintSet("projection", 1, $s++);
        $s++;
        print "$projection_point $ymin\n";
        print "$projection_point $ymax\n";
    }

}

sub PrintPoints {
    my @points = @_;
    my ($last_x, $last_y);
    foreach my $point (sort @points) {
	my ($x,$y) = split /\s+/, $point;
	#if ($y != $last_y) {
	    print "$point\n";
	#}
	$last_x = $x;
	$last_y = $y;
    }
    print "$last_x $last_y\n";
}

sub PrintSet {
    my ($legend, $color_index, $s_index) = @_;
    $s_index = sprintf("%d", $s_index);
    $color_index = sprintf("%d", $color_index);
print<<EOP;
\@    s$s_index hidden false
\@    s$s_index type xy
\@    s$s_index symbol 0
\@    s$s_index symbol size 1.000000
\@    s$s_index symbol color $color_index
\@    s$s_index symbol pattern 1
\@    s$s_index symbol fill color $color_index
\@    s$s_index symbol fill pattern 0
\@    s$s_index symbol linewidth 1.0
\@    s$s_index symbol linestyle 1
\@    s$s_index symbol char 65
\@    s$s_index symbol char font 0
\@    s$s_index symbol skip 0
\@    s$s_index line type 3
\@    s$s_index line linestyle 1
\@    s$s_index line linewidth 1.0
\@    s$s_index line color $color_index
\@    s$s_index line pattern 1
\@    s$s_index baseline type 0
\@    s$s_index baseline off
\@    s$s_index dropline off
\@    s$s_index fill type 2
\@    s$s_index fill rule 0
\@    s$s_index fill color $color_index
\@    s$s_index fill pattern 1
\@    s$s_index avalue off
\@    s$s_index avalue type 2
\@    s$s_index avalue char size 1.000000
\@    s$s_index avalue font 0
\@    s$s_index avalue color $color_index
\@    s$s_index avalue rot 0
\@    s$s_index avalue format general
\@    s$s_index avalue prec 3
\@    s$s_index avalue prepend ""
\@    s$s_index avalue append ""
\@    s$s_index avalue offset 0.000000 , 0.000000
\@    s$s_index errorbar on
\@    s$s_index errorbar place both
\@    s$s_index errorbar color $color_index
\@    s$s_index errorbar pattern 1
\@    s$s_index errorbar size 1.000000
\@    s$s_index errorbar linewidth 1.0
\@    s$s_index errorbar linestyle 1
\@    s$s_index errorbar riser linewidth 1.0
\@    s$s_index errorbar riser linestyle 1
\@    s$s_index errorbar riser clip off
\@    s$s_index errorbar riser clip length 0.100000
\@    s$s_index legend  "$legend"
EOP
}

sub PrintPool {
    my ($color_index, $s_index) = @_;
    $color_index = sprintf("%d", $color_index);
    $s_index = sprintf("%d", $s_index);
    my $legend = "";
print<<EOP;
\@    s$s_index hidden false
\@    s$s_index type xy
\@    s$s_index symbol 0
\@    s$s_index symbol size 1.000000
\@    s$s_index symbol color $color_index
\@    s$s_index symbol pattern 1
\@    s$s_index symbol fill color $color_index
\@    s$s_index symbol fill pattern 4
\@    s$s_index symbol linewidth 1.0
\@    s$s_index symbol linestyle 1
\@    s$s_index symbol char 65
\@    s$s_index symbol char font 0
\@    s$s_index symbol skip 0
\@    s$s_index line type 3
\@    s$s_index line linestyle 1
\@    s$s_index line linewidth 1.0
\@    s$s_index line color $color_index
\@    s$s_index line pattern 4
\@    s$s_index baseline type 0
\@    s$s_index baseline off
\@    s$s_index dropline off
\@    s$s_index fill type 2
\@    s$s_index fill rule 0
\@    s$s_index fill color $color_index
\@    s$s_index fill pattern 4
\@    s$s_index avalue off
\@    s$s_index avalue type 2
\@    s$s_index avalue char size 1.000000
\@    s$s_index avalue font 0
\@    s$s_index avalue color $color_index
\@    s$s_index avalue rot 0
\@    s$s_index avalue format general
\@    s$s_index avalue prec 3
\@    s$s_index avalue prepend ""
\@    s$s_index avalue append ""
\@    s$s_index avalue offset 0.000000 , 0.000000
\@    s$s_index errorbar on
\@    s$s_index errorbar place both
\@    s$s_index errorbar color $color_index
\@    s$s_index errorbar pattern 1
\@    s$s_index errorbar size 1.000000
\@    s$s_index errorbar linewidth 1.0
\@    s$s_index errorbar linestyle 1
\@    s$s_index errorbar riser linewidth 1.0
\@    s$s_index errorbar riser linestyle 1
\@    s$s_index errorbar riser clip off
\@    s$s_index errorbar riser clip length 0.100000
\@    s$s_index legend  "$legend"
EOP
}
