## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;

sub GetTitles {
    my ($type) = @_;
    my $title = "IPv4 allocated /8s";
    $title .= " ($type)" if (defined $type);
    my $subtitle = "RIR whois dumps and IANA table of top-level /8 allocations";
    return ($title, $subtitle);
}

sub GetYAxisLabel() {
    return "allocated /8s";
}

sub GetXAxisLabel() {
    return "dates";
}

sub GetRirPrediction {
    my ($type) = @_;
    my $exponential = "\$y = 2.71828183**(\$a[0] + \$a[1]*\$x)";
    my %first_rir2prediction = %{{
	"APNIC" => {
	    "equation" => $exponential,
	    "a" => [-666.0188,  0.333409],
	    "pool" => [2, 2]
	    },
	"ARIN" => {
	    "equation" => $exponential,
	    "a" => [-326.3702, 0.1641309],
	    "pool" => [3, 2]
	    },
	"LACNIC" => {
	    "equation" => $exponential,
	    "a" => [-1425.157, 0.7110107],
	    #"a" => [-1425.157, 0.0003110107],
	    "pool" => [1, 2]
	    },
	"RIPE" => {
	    "equation" => $exponential,
	    "a" => [ -415.3178, 0.2084018],
	    "pool" => [2, 2]
	    },
	"various" => {
	    "equation" => $exponential,
	    "a" => [-111.7041, 0.05789553],
	    "pool" => [0, 0]
	    }
	}};
    my %specific_rir2prediction = %{{
	"APNIC" => {
	    "equation" => $exponential,
	    "a" => [-712.9462,  0.3567923],
	    "pool" => [2, 2]
	    },
	"ARIN" => {
	    "equation" => $exponential,
	    "a" => [-370.165, 0.1859458],
	    "pool" => [3, 2]
	    },
	"LACNIC" => {
	    "equation" => $exponential,
	    "a" => [-1434.017, 0.7154094],
	    "pool" => [1, 2]
	    },
	"RIPE" => {
	    "equation" => $exponential,
	    "a" => [ -540.5238, 0.2707778],
	    "pool" => [2, 2]
	    },
	"various" => {
	    "equation" => $exponential,
	    "a" => [-127.1083, 0.06557822],
	    "pool" => [0, 0]
	    }
	}};
    my %type2rir2prediction = %{{ 
	"first" => \%first_rir2prediction,
	"specific" => \%specific_rir2prediction
	}};
    return $type2rir2prediction{$type};
}

sub PrintHeader {
    my ($xmin, $xmax, $ymin, $ymax, $type, $log_y) = @_;
    my $xdiff = $xmax - $xmin;
    my $ydiff = $ymax - $ymin;
    $xmin -= $xdiff*.01;
    $ymin -= $xdiff*.01;
    $xmax += $xdiff*.01;
    $ymax += $xdiff*.01;

    my $y_scale = "Normal";
    if (defined $log_y) {
	$y_scale = "Logarithmic";
	$ymin = .1 if ($ymin < 0);
    }

    my ($title, $subtitle) = GetTitles($type);
    my $x_label = GetXAxisLabel();
    my $y_label = GetYAxisLabel();
print<<EOP;
# Grace project file
#
\@version 50114
\@page size 792, 612
\@page scroll 5%
\@page inout 5%
\@link page off
\@map font 0 to "Times-Roman", "Times-Roman"
\@map font 1 to "Times-Italic", "Times-Italic"
\@map font 2 to "Times-Bold", "Times-Bold"
\@map font 3 to "Times-BoldItalic", "Times-BoldItalic"
\@map font 4 to "Helvetica", "Helvetica"
\@map font 5 to "Helvetica-Oblique", "Helvetica-Oblique"
\@map font 6 to "Helvetica-Bold", "Helvetica-Bold"
\@map font 7 to "Helvetica-BoldOblique", "Helvetica-BoldOblique"
\@map font 8 to "Courier", "Courier"
\@map font 9 to "Courier-Oblique", "Courier-Oblique"
\@map font 10 to "Courier-Bold", "Courier-Bold"
\@map font 11 to "Courier-BoldOblique", "Courier-BoldOblique"
\@map font 12 to "Symbol", "Symbol"
\@map font 13 to "ZapfDingbats", "ZapfDingbats"
\@map color 0 to (255, 255, 255), "white"
\@map color 1 to (0, 0, 0), "black"
\@map color 2 to (255, 0, 0), "red"
\@map color 3 to (0, 255, 0), "green"
\@map color 4 to (0, 0, 255), "blue"
\@map color 5 to (255, 255, 0), "yellow"
\@map color 6 to (188, 143, 143), "brown"
\@map color 7 to (220, 220, 220), "grey"
\@map color 8 to (148, 0, 211), "violet"
\@map color 9 to (0, 255, 255), "cyan"
\@map color 10 to (255, 0, 255), "magenta"
\@map color 11 to (255, 165, 0), "orange"
\@map color 12 to (114, 33, 188), "indigo"
\@map color 13 to (103, 7, 72), "maroon"
\@map color 14 to (64, 224, 208), "turquoise"
\@map color 15 to (0, 139, 0), "green4"
\@reference date 0
\@date wrap off
\@date wrap year 1950
\@default linewidth 1.0
\@default linestyle 1
\@default color 1
\@default pattern 1
\@default font 0
\@default char size 1.000000
\@default symbol size 1.000000
\@default sformat "%.8g"
\@background color 0
\@page background fill on
\@timestamp off
\@timestamp 0.03, 0.03
\@timestamp color 1
\@timestamp rot 0
\@timestamp font 0
\@timestamp char size 1.000000
\@timestamp def "Wed Oct  5 17:07:47 2005"
\@r0 off
\@link r0 to g0
\@r0 type above
\@r0 linestyle 1
\@r0 linewidth 1.0
\@r0 color 1
\@r0 line 0, 0, 0, 0
\@r1 off
\@link r1 to g0
\@r1 type above
\@r1 linestyle 1
\@r1 linewidth 1.0
\@r1 color 1
\@r1 line 0, 0, 0, 0
\@r2 off
\@link r2 to g0
\@r2 type above
\@r2 linestyle 1
\@r2 linewidth 1.0
\@r2 color 1
\@r2 line 0, 0, 0, 0
\@r3 off
\@link r3 to g0
\@r3 type above
\@r3 linestyle 1
\@r3 linewidth 1.0
\@r3 color 1
\@r3 line 0, 0, 0, 0
\@r4 off
\@link r4 to g0
\@r4 type above
\@r4 linestyle 1
\@r4 linewidth 1.0
\@r4 color 1
\@r4 line 0, 0, 0, 0
\@g0 on
\@g0 hidden false
\@g0 type XY
\@g0 stacked false
\@g0 bar hgap 0.000000
\@g0 fixedpoint off
\@g0 fixedpoint type 0
\@g0 fixedpoint xy 0.000000, 0.000000
\@g0 fixedpoint format general general
\@g0 fixedpoint prec 6, 6
\@with g0
\@    world xmin $xmin
\@    world xmax $xmax
\@    world ymin $ymin
\@    world ymax $ymax
\@    stack world 0, 0, 0, 0
\@    znorm 1
\@    view xmin 0.150000
\@    view xmax 1.150000
\@    view ymin 0.150000
\@    view ymax 0.850000
\@    title "$title"
\@    title font 0
\@    title size 1.500000
\@    title color 1
\@    subtitle "$subtitle"
\@    subtitle font 0
\@    subtitle size 1.000000
\@    subtitle color 1
\@    xaxes scale Normal
\@    yaxes scale $y_scale
\@    xaxes invert off
\@    yaxes invert off
\@    xaxis  on
\@    xaxis  type zero false
\@    xaxis  offset 0.000000 , 0.000000
\@    xaxis  bar on
\@    xaxis  bar color 1
\@    xaxis  bar linestyle 1
\@    xaxis  bar linewidth 1.0
\@    xaxis  label "$x_label"
\@    xaxis  label layout para
\@    xaxis  label place auto
\@    xaxis  label char size 1.000000
\@    xaxis  label font 0
\@    xaxis  label color 1
\@    xaxis  label place normal
\@    xaxis  tick on
\@    xaxis  tick major .001
\@    xaxis  tick minor ticks 1
\@    xaxis  tick default 6
\@    xaxis  tick place rounded true
\@    xaxis  tick in
\@    xaxis  tick major size 1.000000
\@    xaxis  tick major color 1
\@    xaxis  tick major linewidth 1.0
\@    xaxis  tick major linestyle 1
\@    xaxis  tick major grid off
\@    xaxis  tick minor color 1
\@    xaxis  tick minor linewidth 1.0
\@    xaxis  tick minor linestyle 1
\@    xaxis  tick minor grid off
\@    xaxis  tick minor size 0.500000
\@    xaxis  ticklabel on
\@    xaxis  ticklabel format general
\@    xaxis  ticklabel prec 5
\@    xaxis  ticklabel formula ""
\@    xaxis  ticklabel append ""
\@    xaxis  ticklabel prepend ""
\@    xaxis  ticklabel angle 0
\@    xaxis  ticklabel skip 0
\@    xaxis  ticklabel stagger 0
\@    xaxis  ticklabel place normal
\@    xaxis  ticklabel offset auto
\@    xaxis  ticklabel offset 0.000000 , 0.010000
\@    xaxis  ticklabel start type auto
\@    xaxis  ticklabel start 0.000000
\@    xaxis  ticklabel stop type auto
\@    xaxis  ticklabel stop 0.000000
\@    xaxis  ticklabel char size 1.000000
\@    xaxis  ticklabel font 0
\@    xaxis  ticklabel color 1
\@    xaxis  tick place both
\@    xaxis  tick spec type none
\@    yaxis  on
\@    yaxis  type zero false
\@    yaxis  offset 0.000000 , 0.000000
\@    yaxis  bar on
\@    yaxis  bar color 1
\@    yaxis  bar linestyle 1
\@    yaxis  bar linewidth 1.0
\@    yaxis  label "$y_label"
\@    yaxis  label layout para
\@    yaxis  label place auto
\@    yaxis  label char size 1.000000
\@    yaxis  label font 0
\@    yaxis  label color 1
\@    yaxis  label place normal
\@    yaxis  tick on
\@    yaxis  tick major .001
\@    yaxis  tick minor ticks 1
\@    yaxis  tick default 6
\@    yaxis  tick place rounded true
\@    yaxis  tick in
\@    yaxis  tick major size 1.000000
\@    yaxis  tick major color 1
\@    yaxis  tick major linewidth 1.0
\@    yaxis  tick major linestyle 1
\@    yaxis  tick major grid off
\@    yaxis  tick minor color 1
\@    yaxis  tick minor linewidth 1.0
\@    yaxis  tick minor linestyle 1
\@    yaxis  tick minor grid off
\@    yaxis  tick minor size 0.500000
\@    yaxis  ticklabel on
\@    yaxis  ticklabel format general
\@    yaxis  ticklabel prec 20
\@    yaxis  ticklabel formula ""
\@    yaxis  ticklabel append ""
\@    yaxis  ticklabel prepend ""
\@    yaxis  ticklabel angle 0
\@    yaxis  ticklabel skip 0
\@    yaxis  ticklabel stagger 0
\@    yaxis  ticklabel place normal
\@    yaxis  ticklabel offset auto
\@    yaxis  ticklabel offset 0.000000 , 0.010000
\@    yaxis  ticklabel start type auto
\@    yaxis  ticklabel start 0.000000
\@    yaxis  ticklabel stop type auto
\@    yaxis  ticklabel stop 0.000000
\@    yaxis  ticklabel char size 1.000000
\@    yaxis  ticklabel font 0
\@    yaxis  ticklabel color 1
\@    yaxis  tick place both
\@    yaxis  tick spec type none
\@    altxaxis  off
\@    altyaxis  off
\@    legend on
\@    legend loctype view
\@    legend 0.25, 0.8
\@    legend box color 1
\@    legend box pattern 1
\@    legend box linewidth 1.0
\@    legend box linestyle 1
\@    legend box fill color 0
\@    legend box fill pattern 1
\@    legend font 0
\@    legend char size 1.000000
\@    legend color 1
\@    legend length 4
\@    legend vgap 1
\@    legend hgap 1
\@    legend invert false
\@    frame type 0
\@    frame linestyle 1
\@    frame linewidth 1.0
\@    frame color 1
\@    frame pattern 1
\@    frame background color 0
\@    frame background pattern 0
EOP
}

1;
