#! /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use Time::Local;
use strict;

my %month = BuildMonth();
my $data;
my @events;
AddRfc();
Parse();
PrintEvents();

sub Parse {
    while (<>) {
	s/
(\n){0,1}//g;
	if (/(\d{3}\/\d)\s+(\w{3})\s(\d\d)\s+(.*)/) {
	    my ($block, $month, $year, $name) = ($1, $2, $3, $4);
	    my ($first, $length) = ($block =~ /(\d\d\d)\/(\d)/);
	    $first = sprintf ("%d", $first);
	    $block = $first.".0.0.0/$length";
	    my $time = GetTime($year, $month);
	    if ($name =~ /Returned to IANA\s+([^\s]{3})\s+(\d+)/) {
		my ($month, $year) = ($1, $2);
		my $time = GetTime($year, $month);
		#AddEvent($block, $time, "returned", $name);
	    } elsif ($name =~ /Formerly (.*) - (.{3})\s+(\d\d)/) {
		my ($place, $mon, $yr) = ($1, $2, $3);
		my $new_time = GetTime($yr, $mon);
		if ($place =~ /^IANA/) {
#		    AddEvent($block, $time, "allocated", $name);
		    AddEvent($block, $new_time, "reserved", $place);
		} else {
#		    AddEvent($block, $time, "reserved", $name);
		    AddEvent($block, $new_time, "allocated", $place);
		}
	    } elsif ($name =~ /^IANA - Reserved/) {
		AddEvent($block, $time, "reserved", $name);
	    } elsif ($name =~ /^IANA - Multicast/) {
		AddEvent($block, $time, "allocated", $name);
	    } elsif ($name =~ /IANA - Private Use/) {
		#AddEvent($block, $time, "rfc", $name);
	    } else {
		AddEvent($block, $time, "allocated", $name);
	    }
	}
    }
}

sub AddEvent {
    my ($block, $date, $type, $name) = @_;
    my $org = "various";
    if ($name =~ /^APNIC/) {
	$org = "APNIC";
    } elsif ($name =~ /^ARIN/) {
	$org = "ARIN";
    } elsif ($name =~ /^RIPE/) {
	$org = "RIPE";
    } elsif ($name =~ /^AfriNIC/) {
	$org = "AFRINIC";
    } elsif ($name =~ /^LACNIC/) {
	$org = "LACNIC";
    } elsif ($block eq "14.0.0.0/8") {
	$org = "various";
    } elsif ($name =~ /ulticast/) {
	$org = "IETF multicast";
    } elsif ($name =~ /future use/) {
	$org = "IETF future use";
    } elsif ($name =~ /^IANA/) {
	$org = "IANA";
    } elsif ($name =~ /^RFC/) {
	if ($name =~ /reuseable/) {
	    $org = "IETF reuseable";
	} else {
	    $org = "IETF";
	}
    }
    push @events, {
	"block" => $block,
	"date" => $date,
	"type" => $type,
	"org" => $org,
	"name" => $name
	};
}

sub PrintEvents {
    foreach my $event ( sort {
	    if ($a->{date} == $b->{date}) {
		return $a->{block}<=>$b->{block}
	    }
	    $a->{date}<=>$b->{date}
	    } @events) {
	my ($block, $date, $type, $org, $name) = (
	    $event->{"block"},$event->{"date"}, $event->{"type"},
	    $event->{"org"}, $event->{"name"});
	printf "%-19s| %-9s| %-10s| %-10s| %s\n", 
	    $block, $date, $type, $org, $name;
    }
}
=cut
my $size = 0;
foreach my $year (sort {$a<=>$b} keys %allocate) {
    $size += $allocate{$year};
    print "$year $size\n";
}
=cut

sub BuildMonth {
    my %month;
    my $index = 1;
    foreach my $mon (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)) {
	$month{$mon} = $index++;
    }
    return %month;
}

sub GetTime {
    my ($year, $month) = @_;
    if ($year < 50) {
	$year += 2000;
    } else {
	$year += 1900;
    }
    my $mon = $month{$month};
    my $day = 1;
    foreach my $v ($mon, $day) { 
	$v = "0".$v if ($v < 10); 
    }
    return "$year$mon$day";
    #my ($hour, $min, $sec) = ( 0,0,0 );
    #return Time::Local::timelocal($sec,$min,$hour,$day,$mon,$year);
}

sub AddRfc {
#
#       Summary from RFC 3330
#	    1700    Oct 1994 19941001
#	    1797    Apr 1995 19950401
#	    1918    Feb 1996 19960201
#	    2544    Mar 1999 19990301
#	    3068    Jun 2001 20010601
#	    3171    Aug 2001 20010801
#	    3330    Sep 2002 20020901
#
#   0.0.0.0/8            "This" Network                 [RFC1700, page 4]
#   10.0.0.0/8           Private-Use Networks                   [RFC1918]
#   14.0.0.0/8           Public-Data Networks         [RFC1700, page 181]
#   39.0.0.0/8           Reserved but subject
#                           to allocation                       [RFC1797]
#   127.0.0.0/8          Loopback                       [RFC1700, page 5]
#   128.0.0.0/16         Reserved but subject
#                           to allocation                             --
#   169.254.0.0/16       Link Local                                   --
#   172.16.0.0/12        Private-Use Networks                   [RFC1918]
#   191.255.0.0/16       Reserved but subject
#                           to allocation                             --
#   192.0.0.0/24         Reserved but subject
#                           to allocation                             --
#   192.0.2.0/24         Test-Net
#   192.88.99.0/24       6to4 Relay Anycast                     [RFC3068]
#   192.168.0.0/16       Private-Use Networks                   [RFC1918]
#   198.18.0.0/15        Network Interconnect
#                           Device Benchmark Testing            [RFC2544]
#   223.255.255.0/24     Reserved but subject
#                           to allocation                             --
#   224.0.0.0/4          Multicast                              [RFC3171]
#   240.0.0.0/4          Reserved for Future Use        [RFC1700, page 4]
#
    AddEvent("0.0.0.0/8", 19941001, "allocated", "RFC1700");
    AddEvent("10.0.0.0/8", 19960201, "allocated", "RFC1918");
    AddEvent("14.0.0.0/8", 19941001, "allocated", "RFC1700");
#AddEvent("39.0.0.0/8", 19950401, "allocated", "RFC1797 reuseable");
#AddEvent("39.0.0.0/8", 20020901, "returned", "RFC3330");
    AddEvent("127.0.0.0/8", 19941001, "allocated", "RFC1700");
#AddEvent("128.0.0.0/16", 19941001, "allocated", "RFC1700 reuseable");
#AddEvent("128.0.0.0/16", 20020901, "returned", "RFC3330");
    AddEvent("169.254.0.0/16", 19941001, "allocated", "RFC1700");
    AddEvent("172.16.0.0/12", 19960201, "allocated", "RFC1918");
#AddEvent("191.255.0.0/16", 19960201, "allocated", "RFC1918 reuseable");
#AddEvent("191.255.0.0/16", 20020901, "returned", "RFC3330");
#AddEvent("192.0.0.0/24", 19960201, "allocated", "RFC1918 reuseable");
#AddEvent("192.0.0.0/24", 20020901, "returned", "RFC3330");
    AddEvent("192.0.2.0/24", 19960201, "allocated", "RFC1918");
    AddEvent("192.88.99.0/24", 20010601, "allocated", "RFC3068");
    AddEvent("192.168.0.0/16", 19960201, "allocated", "RFC1918");
    AddEvent("198.18.0.0/15", 19990301, "allocated", "RFC2544");
#AddEvent("233.255.255.0/24", 19990301, "allocated", "RFC2544 reuseable");
#AddEvent("233.255.255.0/24", 20020901, "returned", "RFC3330");
    #AddEvent("224.0.0.0/4", 19810901, "allocated", "RFC multicast");
AddEvent("240.0.0.0/4", 19810901, "allocated", "future use");
}


