#!  /usr/local/pkg/perl-5.8.6/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
use Socket;
use Socket6;
use Getopt::Std;
use Net::IP;

my ($executable) = ($0 =~ /([^\/]+)$/);
my $USAGE = "usage:$executable [-h] whois.database";
sub USAGE() { return $USAGE};
sub Help() {
print STDERR<<EOP;
$USAGE
    This program parses the whois database dumps and prints out the 
    current allocations of inet4 and inet6.

    Not that you should first call splitWhois.pl to break the file
    into it's parts and call this on the individual parts.  In the 
    case of arin you need to feed this program first the OrgIDs
    and next the relative address allocations since it gets
    the country from the OrgIDs file.

    h - this message

EOP
}
my $ALL_ONES = unpack("N",inet_aton("255.255.255.255"));
my $ALL_ZEROS = unpack("N",inet_aton("0.0.0.0"));

my %opts;
getopts("s:ho", \%opts);
if (defined $opts{h}) {
    Help();
    exit;
}

my $TODAY = Time2Date(time);

my $include_org = defined $opts{o};
my $skip_to = $opts{s};

# org is not currently supported.
#my @args = qw(inet netrange netname org country status changed source);
my @args = qw(inet netrange netname country status changed source);
push @args, "orghandle" if $include_org;
print join(" ",("#",@args)),"\n";

my %len4size = BuildLength4Size();
my %len4mask = BuildLength4Mask();

my $entrynum = 0;
my $linenum = 0;
ProcessObjects();

my %org2country;
sub ProcessObjects {
    my ($type, $arg2value);
    my $skipping;
    if (defined $skip_to) {
	$skipping = 1;
	print "# skipping to \"$skip_to\"\n";
    }
    my ($arg_last, $value_last);
    while (<>) {
	$linenum++;
	undef $skipping if (/$skip_to/);
	next if (defined $skipping);
#if ($linenum > 719010 && $linenum < 719018) {
#print STDERR ;
#}
	if (/^\s*$/) {
	    if (defined $arg_last) {
		($type, $arg2value) = Parse($type, $arg_last, $value_last, 
		    $arg2value);
		undef $arg_last;
	    }
	    if ($type eq "inet") {
		PrintInet($arg2value);
	    }
	    undef $type;
	} elsif (/^([^:]+):\s*(.*)/) {
	    my ($arg,$value) = ($1, $2);
	    if (defined $arg_last) {
		($type, $arg2value) = Parse($type, $arg_last, $value_last, 
		    $arg2value);
		undef $arg_last;
	    }
	    $arg_last = $arg;
	    $value_last = $value;
	} elsif ((/^\s*([\s].*)/) || (defined $arg_last)) {
	    my $value = $1;
	    $value =~ s/\s+$//;
	    $value_last .= " ".$value;
	}
    }
    if (defined $arg_last) {
	($type, $arg2value) = Parse($type, $arg_last, $value_last, $arg2value);
	undef $arg_last;
    }
    if ($type eq "inet") {
	PrintInet($arg2value);
    }
}

sub Parse {
    my ($type, $arg, $value, $arg2value) = @_;
    foreach my $v ($arg, $value) { 
	$v =~ s/^\s*//g;
	$v =~ s/\s*$//g;
    }
    if (!defined $type) {
	($type, $arg2value) = CreateArgValue($arg, $value);
	$entrynum++;
	print STDERR "$linenum:$entrynum arg:$arg value:$value\n" 
	    if ($entrynum%10000 == 0);
	unless ($type eq "inet" || $type eq "orgid") {
	    print STDERR "Unknown type $linenum $_";
	}
    } else {
	if ($type eq "orgid") {
	    ParseOrgId($arg, $value, $arg2value);
	} elsif ($type eq "inet") {
	    ParseInet($arg, $value, $arg2value);
	}
    } 
    return ($type, $arg2value);
}

sub CreateArgValue {
    my ($arg, $value) = @_;
    my ($type, %arg2value);
    if ($arg eq "OrgID") {
	$arg2value{"orgid"} = [$value];
	$type = "orgid";
    } 
    
    if ($arg eq "NetHandle" || $arg eq "V6NetHandle"
	|| $arg eq "inetnum" || $arg eq "inet6num") {
	$type = "inet";
	$arg2value{"nethandle"} = [$value];
	my ($inet, $netrange);
	if ($arg eq "NetHandle") {
	    $inet = "inetnum";
	} elsif ( $arg eq "V6NetHandle") {
	    $inet = "inet6num";
	} elsif ($value =~ /\:/) {
	    $inet = "inet6num";
	    $netrange = $value;
	} else {
	    $inet = $arg;
	    $netrange = $value;
	}
	$arg2value{"inet"} = [$inet];
	if (defined $netrange) {
	    $arg2value{"netrange"} = [$netrange];
	}
    }
    return ($type, \%arg2value);
}

sub ParseOrgId {
    my ($arg, $value, $arg2value) = @_;
    if ($arg eq "Country" && $value =~ /(\w\w)/) {
	$org2country{$arg2value->{"orgid"}[0]} = $1;
    }
}

sub ParseInet {
    my ($arg, $value, $arg2value) = @_;
    $arg =~ y/A-Z/a-z/;
    $value =~ s/#.*//;
    $value =~ s/^\s+//;
    $value =~ s/\s+$//;
    $value =~ s/\s*\t\s*/ /g;
    #################################################
    # These are for {6}?NetHandles
    #################################################
    if ($arg eq "orgid") {
	if (defined $value) {
	    push @{$arg2value->{$arg}}, $value;
	    push @{$arg2value->{"orghandle"}}, "orgid:$value" if $include_org;
	}
	$value = $org2country{$value};
	$arg = "country";
    } elsif ($arg eq "nettype" || $arg eq "status") {
	$arg = "status";
	$value =~ y/A-Z/a-z/;
    #################################################
    # Below is for both
    #################################################
    } elsif ($arg eq "regdate" || $arg eq "updated" || $arg eq "changed") {
	$arg = "changed";
	my $orginal = $value;
	$value =~ s/-//g;
	$value =~ s/\s*#.*//g;
	($value) = ($value =~ /(\d+)[^\d]*$/);
	if ($value =~ /^\d{6}$/) {
	    if ($value =~ /^0/) {
		$value = "20".$value;
	    } elsif ($value =~ /^[98]/) {
		$value = "19".$value;
	    } elsif (($value =~ /^19/ || $value =~ /^20/)) {
		while (length($value) < 8) {
		    $value .= "01";
		}
	    } else {
		die("Failed to parse date:$value from\"$orginal\"");
	    }
	    #if ($value < 19810101 || $value > $TODAY ) {
		#print STDERR "$orginal -> $value\n";
	    #}
	} elsif (!($value =~ /\d{8}/)) {
	    undef $value;
	}
    } elsif ($arg eq "source") {
	$value =~ y/a-z/A-Z/;
    } elsif ($arg eq "admin-c") {  # APNIC & RIPE
        push @{$arg2value->{"orghandle"}}, "admin-c:$value" if $include_org;
    } elsif ($arg eq "owner-c") {  # LACNIC -- not always there
        push @{$arg2value->{"orghandle"}}, "owner-c:$value" if $include_org;
    }
	    
    if ($value =~ /[^\s]/) {
	my $found;
	foreach my $v (@{$arg2value->{$arg}}) {
	    if ($v eq $value) {
		return;
	    }
	}
	push @{$arg2value->{$arg}}, $value;
    }
}

my %parent2country;
sub PrintInet {
    my ($arg2value) = @_;
    my %arg2value = %$arg2value;
    if (defined $arg2value->{parent}) {
	my $country = $parent2country{$arg2value{parent}[0]};
	if (defined $country) {
	    push @{$arg2value{country}}, $country;
	}
    }
    if (defined $arg2value{country}) {
	my $nethandle = $arg2value{"nethandle"};
	$parent2country{$nethandle} = $arg2value{country}[0];
    } 
    if ($arg2value{source}[0] eq "ARIN") {
	my $orgid = $arg2value{orgid}[0];
	if ($orgid eq "RIPE" || $orgid eq "AFRINIC" || $orgid eq "LACNIC") {
	    $arg2value{status} = ["rir"];
	}
    }
    if (!defined $arg2value{country}) {
=cut
	print "# ";
	foreach my $arg (qw(orgid parent)) {
	    if (defined $arg2value{$arg}) {
		print "$arg:",join(",",@{$arg2value{$arg}}),"\t";
	    }
	}
	print "\n";
=cut
    }

    my @values;
    foreach my $arg (@args) {
	my $values = $arg2value{$arg};
	if (defined $values) {
	    if ($arg eq "netrange") {
		foreach my $value (@$values) {
		    $value = ConvertRange($value);
		}
	    }
	    push @values, join(",",@$values);
	} else {
	    push @values, "";
	}
    }
    print join("\t",@values),"\n";
}

sub ConvertRange {
    my ($range) = @_;
    my ($from,$to) = split /\s*-\s*/,$range;
    my @prefixes;
    if (!defined $to && $range =~ /\//) {
	my ($addr, $len) = split /\//,$range;
	if ($addr =~ /\./) {
	    $addr = CleanUp4($addr);
	} 
	@prefixes = ($addr."/".$len);
    } else {
	if (!defined $to) {
	    $from = $to = $range;
	}
	if ($from =~ /^[\d\.]+$/) {
	    @prefixes = ConvertRange4($from,$to);
	} else {
	    @prefixes = ConvertRange6($from,$to);
	}
    } 
    return join(",",@prefixes);
}

sub ConvertRange4 {
    my ($from,$to) = @_;
    $from = CleanUp4($from);
    $to = CleanUp4($to);
    my $ip_range = new Net::IP("$from - $to");
    return $ip_range->find_prefixes();
=cut
    my $current = unpack("N", inet_aton($from));
    my $target = unpack("N", inet_aton($to));
    my @prefixes;
    do {
	my $len = 32;
	my $shift = 1;
	my $current_inet = pack("N",$current);
	my $inet = $current_inet;
	my $masked = $len4mask{$len-1} & $inet;
	while ($current+$len4size{$len-1}-1 <= $target
	    && $masked eq $inet) {
	    $len--;
	    $masked = $len4mask{$len-1} & $current_inet;
	}
	$masked = $len4mask{$len} & $current_inet;
	push @prefixes, inet_ntoa($current_inet)."/".$len;
	$current += $len4size{$len};
#print "len:$len\n";
#print "masked:",inet_ntoa($masked),"\n";
#print "current_inet:",inet_ntoa($current_inet),"\n";
#print "currnt:$current    target:$target\n";
    } while ($current <= $target);
    CheckRange4($from, $to, @prefixes);
    return @prefixes;
=cut
}
    sub CleanUp4 {
	my ($input) = @_;
	my @values = split /\./, $input;
	foreach my $index (0..3) {
	    $values[$index] = 0 unless (defined $values[$index]);
	    $values[$index] =~ sprintf("%i",$values[$index]);
	}
	return join(".",@values);
    }

    sub CheckRange4 {
	my ($from, $to, @prefixes) = @_;
	my $last = inet_aton($from);
	my $size = unpack("N",inet_aton($to)) -
		   unpack("N",inet_aton($from)) + 1;
	my $total;
	my @errors;
#printf ("%3s %15s %15s\n", "len", "total", "size");
	foreach my $prefix_len (@prefixes) {
	    my ($prefix, $len) = split /\//, $prefix_len;
	    $total += $len4size{$len};
#printf ("%3d %15d %15d\n", $len, $total, $len4size{$len});
	    my $prefix_inet = inet_aton($prefix);
	    if ($prefix_inet ne $last) {
		push @errors, "Wanted $last found $prefix";
	    }
	    $last = pack("N", $len4size{$len} + unpack("N",$prefix_inet) );
	}
	if ($total != $size) {
	    push @errors, "Wanted number of Address $size != $total";
	}
	if ($#errors >= 0) {
	    die(
		"$from -> $to ",join(",", @prefixes)."\n\t".
		join("\n\t",@errors)."\n");
	}
#print "\n";
    }
	    

sub ConvertRange6 {
    my ($from,$to) = @_;
    my $ip_range = new Net::IP("$from - $to");
    return $ip_range->find_prefixes();
#print "$from - $to\n";
=cut
    my @current = unpack("NNNN",inet_pton(AF_INET6,$from));
    my @target = unpack("NNNN",inet_pton(AF_INET6,$to));
    
    my @prefixes;
    my $finished;
    while (1) {
        my $i = 3;
        while ($i > 0 && ($current[$i] eq $ALL_ZEROS) 
            && ($target[$i] eq $ALL_ONES) ) {
            $i--;
        }
        my $len = 0;
#printf "    %16s %16s\n",inet_ntoa(pack("N",$current[$i])),inet_ntoa(pack("N",$target[$i]));
	do {
            $len = 0;
            my $shift = 1;
            while (($current[$i]+($shift*2)-1 <= $target[$i])&&($len < 32)) {
                $len++;
                $shift *= 2;
            }
            if ($len < 32) {
                push @prefixes, inet_ntop(AF_INET6,pack("NNNN",@current))
                    ."/".(32-$len + $i*32);
                $current[$i] += $shift;
            } else {
                last;
            }   
        } while ($current[$i] <= $target[$i]);
        if ($len < 32 && $current[$i] >= $target[$i]) {
            last;
        }   
        $current[$i] = $ALL_ZEROS;
        $i--;
    }
#print "\t",join(",",@prefixes),"\n";
    return @prefixes;
=cut
}

sub BuildLength4Size {
    my %len4size;
    foreach my $i (1..32) {
        $len4size{$i} = 1<<(32-$i);
    }
    $len4size{0} = 1+0xFFFFFFFF;
    return %len4size;
}

sub BuildLength4Mask {
    my %len4mask;
    foreach my $i (1..32) {
        $len4mask{$i} = pack("N",0xFFFFFFFF<<(32-$i));
    }
    return %len4mask;
}

sub Time2Date {
    my ($time) = @_;
    my ($sec, $min, $hour, $day, $mon, $year) = localtime($time);
    $mon++;
    $year += 1900;
    foreach my $var ($day, $mon) {
	$var = "0".$day if ($var < 10);
    }
    return "$year$mon$day";
}

