#! /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use Compress::Zlib;
use Time::Local;
use Digest::MD5;
use LWP::UserAgent;
use HTTP::Response;
use HTTP::Request;
use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$0 [-d directory] [-h] URL(s)\n";
sub Help() {
print STDERR <<EOP;
$usage;
    This downloads whos stats records and checks the md5 hash found 
    in the URL.
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my $PATH = ".";
$PATH = $opts{d} if (defined $opts{d});
OpenLog($PATH);
my $UserAgent = new LWP::UserAgent;
foreach my $URL (@ARGV) {
    GetStats($PATH,$URL);
}

sub GetStats {
    my ($PATH, $URL) = @_;
    unless (-d $PATH) {
	die("Unable to find $PATH");
    }
    #my @lines = (<>);
    #my $text = join("",@lines);
    my $text = GetUrl($URL);
    my @urls = ParseFTP($URL,$text);
    my %files;
    foreach my $url (@urls) {
	my $temp = $url;
	$temp =~ s/\.gz$//g;
	$temp =~ s/\.bz2$//g;
	if ($temp =~ /([^\/]+\d{8})(\.([^\/]+)){0,1}$/) {
	    my ($file, $type) = ($1, $3);
	    $files{$file}{$type} = $url;
	}
    }
    my $undef;
    foreach my $file (sort {$a<=>$b} keys %files) {
	my $file_url = $files{$file}{$undef};
	my $md5_url = $files{$file}{"md5"};
	if (my ($server,$year) = ($file =~ /([^-]+)[-\.](\d{4})/)) {
	    GetFile($PATH, $year,$server,$file, $file_url, $md5_url);
	}
    }
}

sub GetFile {
    my ($PATH, $year,$server,$file, $file_url, $md5_url) = @_;
    my $path = $PATH;
    foreach my $dir ($year,$server) {
	$path .= "/$dir";
	unless (-d $path) {
	    mkdir ($path, 0774);
	}
    }
	
    $file =~ s/gz$//;
    my $fullname = "$PATH/$year/$server/$file";
    if (-f $fullname) {
	return;
    }
    my $num_tries = 0;
    my ($success, $file_content);
    do {
	print "$file_url -> $fullname\n";
	$file_content = GetUrl($file_url);
	unless (defined $file_content) { return; }
	if (defined $md5_url) {
	    my $md5_content = GetUrl($md5_url);
	    unless (defined $md5_content) { return; }
	    my ($md5);
	    if ($md5_content =~ /MD5 \([^\)]+\)\s+=\s+([^\s]+)/) {
		$md5 = $1;
	    } elsif ($md5_content =~ /^\s*([^\s]+)\s+delegated-afrinic-/) {
		$md5 = $1;
	    }
		
	    if (Digest::MD5::md5_hex($file_content) eq $md5) {
		$success = 1;
	    } else {
		undef $success;
	    }
	} else {
	    $success = 1;
	}
	$num_tries++;
    } while (!defined $success && $num_tries < 3);

    if (defined $success) {
	open(OUT, ">$fullname") || die("Unable to open $fullname:$!");
	print OUT $file_content;
	close OUT;
    } else {
	my $error = "ERROR:$file_url did not match md5 tried $num_tries";
	print STDERR $error,"\n";
	PrintLog($error);
    }
}

sub GetUrl {
    my ($URL) = @_;
    my $UserAgent = new LWP::UserAgent;
    my $req = new HTTP::Request GET => $URL;
    my $res = $UserAgent->request($req);
    if ($res->is_error()) {
	print STDERR $res->error_as_HTML(),"\n";
	return;
    }
    my $content = $res->content();
    if ($URL =~ /.gz$/) {
	$content = Compress::Zlib::memGunzip($content);
    } elsif ($URL =~ /.bz2$/) {
	my $temp_file = "temp.".$$.".bz2";
	open (TEMP,">$temp_file");
	print TEMP $content;
	close TEMP;
	system("bunzip2 $temp_file");
	$temp_file =~ s/\.bz2$//g;
	open (TEMP,"<$temp_file");
	$content = join("",(<TEMP>));
	close TEMP;
	unlink $temp_file;
    }
    return $content;
}

sub ParseFTP {
    my ($URL,$text) = @_;
    my $parser = new MyParser;
    $parser->parse($text);
    my @urls = $parser->Urls();
    if ($#urls > 0) {
	return @urls;
    } else {
	return ParseFtpNonHtml($URL,$text);
    }
}

{
    package MyParser;
    use base 'HTML::Parser';

    my @urls;
    my $name_inside;
    sub start {
	my($self, $tagname, $attr, $attrseq, $origtext) = @_;
	if ($tagname eq "a") {
	    my $url = $attr->{"href"};
	    push @urls, $url;
	}
    }
    sub end {
	my($self, $tagname, $origtext) = @_;
    }

    sub text {
	my($self, $origtext, $is_cdata) = @_;
    }
    sub Urls {
	return @urls;
    }
}

sub ParseFtpNonHtml {
    my ($URL, $text) = @_;
    my @urls;
    foreach my $line (split /\n/, $text) {
	if ($line =~ /^[\-rwx]+.*(delegated-[^\s]+)/) {
	    push @urls, "$URL/$1";
	}
    }
    return @urls;
}

sub DownLoadFile {
    my ($url_struct) = @_;
}

sub OpenLog {
    my ($path) = @_;
    my $log_file = "$path/log";
print "log_file:$log_file\n";
    open(LOG,">>$log_file") || die("Unable to open log file $log_file:$!");
}

sub PrintLog {
    my ($msg) = @_;
    my ($sec, $min, $hour, $day, $mon, $year) = localtime(time);
    $mon++;
    $year += 1900;
    foreach my $value ($sec, $min, $hour, $day, $mon) {
	$value = "0".$value if ($value < 10);
    }
    print LOG "$year$mon$day $hour:$min:$sec $msg\n";
}
