#!/usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 


use Getopt::Std;
use Net::IP;

use strict;

my $usage = "usage:$0 [-heort] inet6num\n";
sub Help() {
    print <<EOP;
$usage
    h - this message
    e - output events rather than the final xmgrace input data
    o - print all overlapping prefixes
    r - exclude reserved bits when measuring the size of prefixes
        in the SubTLA space (2001::/16); without -r, reserved bits
        are treated like any other bits subject to allocation to end networks
    t - input files are whois 'stats' files rather than 'inet6num' files
EOP
}

my %opts;
unless (getopts("heort",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my $output_events = defined $opts{'e'};
my $print_all_overlaps = defined $opts{'o'};
my $count_subtla_reserved_bits = defined $opts{'r'};
my $stats_file_input = defined $opts{'t'};

my $UNICAST_PREFIX = new Net::IP("2000::/3");
my $RESERVED_PREFIX = new Net::IP("2000::/16");
my $SUB_TLA_PREFIX = new Net::IP("2001::/16");
my $SIX2FOUR_PREFIX = new Net::IP("2002::/16");
my $SIXBONE_PREFIX = new Net::IP("3FFE::/16");

my %prefix;
my %counts;
my @info;
ParseWhoisEvents();

my @sorted = sort byprefix @info;
=cut
    print STDERR "DEBUG: sorted prefixes:\n";
    map { print "\t", $_->{inet}->print(), "\n" } @sorted;
=cut

RemoveCounted();
#PrintRootPrefixes();
CountRootPrefixes();


sub ParseWhoisEvents {
    my $linenum = 0;
    while (<>) {
        chomp;
        next if /^\#/;
        next if /^\s*$/;

	my @values;
	my @prefixes;
	my ($family, $country, $type, $date, $rir);

	if ($stats_file_input) {
	  next if /^\d/;  # header line
	  next if /summary/;
	  next unless /ipv6/;

	  my ($start, $length);
	  ($rir, $country, $family, $start, $length, $date, $type)= split /\|/;

	  $rir =~ s/ripencc/RIPE/;
	  $rir = uc $rir;
	  $family =~ s/ipv(.)/inet\1num/;
	  @prefixes = ( "$start/$length" );
	}
	else {
	  @values = split /\t/;
	  foreach my $value (@values) { $value =~ s/^\s+//; $value=~ s/\s+$//;}
	  my ($prefixes, $org);
	  ($family, $prefixes, $org, $country, $type, $date, $rir) = @values;
	  $rir =~ s/#.*//;
	  $rir =~ y/a-z/A-Z/;

	  next if ($prefixes eq "0::/0");  # APNIC and RIPE

	  # keep AFRINIC-NET-TRANSFERRED-20050223 in RIPE records

	  # keep assignments to RIPE as business (vs. RIR) in RIPE records:
	  #  RIPE-NCC-IXP-20010913
	  #  RIPE-NCC-IPv6
	  #  RIPE-NCC  (3 of these)
	  #  RIPE-MEETING
	  #  RIPE-TOKYO (in APNIC)
	  #  STAFF1-RIPE-NCC, STAFF2-RIPE-NCC, ..., STAFF5-RIPE-NCC

	  # IANA allocations to RIPE
	  next if ($org =~ /^EU-ZZ-/ && $rir eq "RIPE");

	  # ARIN seems to have transferred 2001:4300::/26 to RIPE
	  # (from 2001:4300:: to 2001:437F:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF).
	  # ARIN NetName:'s RIPE1, ..., RIPE8
	  next if ($org =~ /^RIPE[1-8]$/ && $rir eq "ARIN");

	  # IANA allocations to ARIN
	  next if ($org =~ /^ARIN-00[1-4]$/ && $rir eq "ARIN");
	  next if ($org =~ /^ARIN-260[048C]$/ && $rir eq "ARIN");

	  # keep assignments to APNIC as business (vs. RIR) in APNIC records:
	  #  APNIC-IXP-2002 (in RIPE)
	  #  APNIC-IPV6-REVERSE-DNS-INFRASTRUCTURE
	  #  APNIC-AP-IX-ASSIGNMENT
	  #  APNIC-AP-V6-20030124

	  # IANA allocations to APNIC
	  next if ($org =~ /^APNIC-AP-ALLOCATED-PORTABLES[1-6]$/
		   && $rir eq "APNIC");
	  next if ($org eq "APNIC-AP-ALLOCATED-PORTABLE" && $rir eq "APNIC");

	  # RIPE seems to have given 2001:2000::/20 (one of the top-level
	  # blocks allocated to RIPE by IANA) to Telia for further LIR
	  # allocations.
	  next if ($org eq "EU-TELIANET-20040510" && $rir eq "RIPE");

	  # RIPE duplicates the APNIC record for 2001:07FA::/32 (APNIC-IXP-2002)
	  # (called APNIC-AP-IX-ASSIGNMENT in APNIC).
	  next if ($org eq "APNIC-IXP-2002" && $rir eq "RIPE");

	  @prefixes = split /,/, $prefixes;
	  ($date) = (sort {$a<=>$b} split /,/, $date)[0];
	  next unless ($date=~ /\d{8}/);
	}

	foreach my $prefix (@prefixes) {
	    my $inet = new Net::IP($prefix, 6);
	    unless (defined $inet) {
	      print STDERR "ERROR: line $linenum: malformed prefix '$prefix'\n";
	      next;
	    }

	    unless (within_prefix($UNICAST_PREFIX, $inet)) {
	      print STDERR "warning: line $linenum: prefix '$prefix' not in global unicast space (2000::/3); skipping\n";
	      next;
	    }

	    if (within_prefix($RESERVED_PREFIX, $inet)) {
	      print STDERR "warning: line $linenum: prefix '$prefix' in reserved global unicast space (2000::/16); skipping\n";
	      next;
	    }
	    elsif (within_prefix($SIX2FOUR_PREFIX, $inet)) {
	      print STDERR "warning: line $linenum: prefix '$prefix' in 6to4 space (2002::/16); skipping\n";
	      next;
	    }
	    elsif (within_prefix($SIXBONE_PREFIX, $inet)) {
	      print STDERR "warning: line $linenum: prefix '$prefix' in 6Bone space (3FFE::/16); skipping\n";
	      next;
	    }

	    # size as positive or negative number of bits difference from 48;
	    # negative means longer than /48; positive means shorter than 48;
	    # 1 / (2 ** $size) is the block size in terms of a single /48
	    my $size = 0;
            my $length = $inet->prefixlen();

	    if ($length > 64) {
	      print STDERR "warning: line $linenum: '$prefix' has a prefix length > 64; skipping\n";
	      next;
	    }
	    elsif ($length >= 48) {
	      $size = 48 - $length;
	    }
	    else { # if $length < 48
	      if ($length < 16) {
		print STDERR "warning: line $linenum: '$prefix' has a prefix length < 16; assuming 16\n";
		$length = 16;
	      }

	      if ($count_subtla_reserved_bits) {
		unless ($inet->overlaps($SUB_TLA_PREFIX)
			== $IP_A_IN_B_OVERLAP) {
		  if ($length < 24) {
		    print STDERR "warning: line $linenum: '$prefix', which is not in the sub-TLA block, has a prefix length < 24; excluding reserved bits from counts by assuming /24\n";
		    $length = 24;
		  }
		}
	      }

	      $size = 48 - $length;
	    }

	    ++$counts{$size};

	    my $info = {
#		"org" => $org, 
		"family" => $family,
		"inet" => $inet,
		"country" => $country,
		"type" => $type,
		"date" => $date,
		"rir" => $rir,
		"size" => $size,
		"root_prefix" => 1,
		};
	    push @info, $info;

	    if ($output_events) {
	      print "$prefix|$date|$type|$rir|$rir\n";
	    }
	}
	$linenum++;
    }
}

sub RemoveCounted {
    my @stack;
    foreach my $info (@sorted) {
	my $inet = $info->{inet};
	while ($#stack >= 0
	       && $inet->overlaps($stack[0]{inet}) == $IP_NO_OVERLAP) {
#print STDERR "DEBUG: popping ", $stack[0]{inet}->print(), " which doesn't overlap ", $inet->print(), "\n";
	    shift @stack;
	}

	foreach my $inet_on_stack (map { $_->{inet} } @stack) {
	  unless ($inet->overlaps($inet_on_stack) == $IP_A_IN_B_OVERLAP) {
	    print STDERR "ERROR: ", $inet_on_stack->print(), " doesn't completely enclose ", $inet->print(), "\n";
	  }
	}

	if ($#stack >= 0) {
	  print STDERR "".(scalar @stack)." OVERLAP: ", $inet->print(), "\n\t",
	    join("\n\t", map { $_->{inet}->print() } @stack), "\n"
	      if $print_all_overlaps || @stack >= 3;

	    $info->{root_prefix} = 0;
	    if ($stack[0]{date} <= $info->{date}) {
		$info->{size_exclude} .= " " . $info->{size};
	    } else {
		$stack[0]{size_exclude} .= " " . $info->{size};
	    }
	}
	unshift @stack, $info;
    }
}


sub byprefix {
  my $a_inet = $a->{inet};
  my $b_inet = $b->{inet};

  # bincomp() only compares the first address in each prefix so we
  # must separately compare the prefix length to order prefixes
  # from largest to smallest (e.g., 2001::/16 before 2001::/32).
  #
  # bincomp() doesn't support "eq".

  return -1 if $a_inet->bincomp('lt', $b_inet);
  return 1 if $a_inet->bincomp('gt', $b_inet);
  return $a_inet->prefixlen() <=> $b_inet->prefixlen();
}


sub PrintRootPrefixes {
  foreach my $info (grep { $_->{root_prefix} } @sorted) {
    print STDERR "ROOT: ", $info->{inet}->print(), "\n";
  }
}


sub CountRootPrefixes {
  my %root_counts;
  foreach my $info (grep { $_->{root_prefix} } @sorted) {
    ++$root_counts{$info->{size}};
  }

  my @lengths = sort {$a<=>$b} keys %root_counts;
  my $min_length = $lengths[0];

  foreach my $k (@lengths) {
    print STDERR $k, " (/", 48 - $k, ")\t", $root_counts{$k}, "\n";
  }

  print STDERR "\n", ("=" x 70), "\n";

  my %counts_norm = %root_counts;

  for (my $k = $min_length; $k <= 31; $k++) {
    next unless defined $counts_norm{$k};

    my $v = $counts_norm{$k};
    my $vdiv2 = int($v / 2);
    my $vmod2 = $v % 2;
    $counts_norm{$k + 1} += $vdiv2;
    $counts_norm{$k} = $vmod2;
  }

  foreach my $k (sort {$a<=>$b} keys %counts_norm) {
    my $v = $counts_norm{$k};
    next if $v == 0;
    print STDERR $k, " (/", 48 - $k, ")\t$v\n";
  }

  my $total_slash48s = 0;
  my $total_slash32s = 0;
  foreach my $k (sort {$a<=>$b} keys %counts_norm) {
    my $v = $counts_norm{$k};
    next if $v == 0;

    $total_slash48s += $v * 2 ** $k;
    $total_slash32s += $v * 2 ** ($k - 16);
  }

  print STDERR "(root prefixes) total /48-equivalent coverage: $total_slash48s\n";

  print STDERR "(root prefixes) total /32-equivalent coverage: $total_slash32s\n";

  my $equiv_prefix_length = 32 - (log($total_slash32s) / log(2));
  print STDERR "(root prefixes) equivalent prefix length: $equiv_prefix_length\n";
}


sub within_prefix
{
  my ($pref_a, $pref_b) = @_;

  my $result = $pref_a->overlaps($pref_b);
  return undef unless defined $result;
  return ($result == $IP_B_IN_A_OVERLAP || $result == $IP_IDENTICAL);
}
